/*  definitions.go - character definitions for go.jp flashcard game
    Copyright (C) 2024 hoff.industries

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>. */

package gojpdefinitions

type RomajiCharacter struct {
	Character string
	Family string
	HanDakuten bool
	ValidRomaji []string
}

func Romaji2Heap(character RomajiCharacter) *RomajiCharacter{
	var heapCharacter = new(RomajiCharacter)

	heapCharacter.Character = character.Character
	heapCharacter.Family = character.Family
	heapCharacter.ValidRomaji = make([]string, len(character.ValidRomaji))

	for i := 0; i < len(character.ValidRomaji); i++ {
		heapCharacter.ValidRomaji[i] = character.ValidRomaji[i]
	}

	return heapCharacter
}

var ROMAJI_HIRAGANA = map[string][]RomajiCharacter{
	"a": []RomajiCharacter{
		RomajiCharacter{
			"あ",
			"a",
			false,
			[]string{"a"},
		},
		RomajiCharacter{
			"か",
			"a",
			false,
			[]string{"ka"},
		},
		RomajiCharacter{
			"さ",
			"a",
			false,
			[]string{"sa"},
		},
		RomajiCharacter{
			"た",
			"a",
			false,
			[]string{"ta"},
		},
		RomajiCharacter{
			"な",
			"a",
			false,
			[]string{"na"},
		},
		RomajiCharacter{
			"は",
			"a",
			false,
			[]string{"ha"},
		},
		RomajiCharacter{
			"ま",
			"a",
			false,
			[]string{"ma"},
		},
		RomajiCharacter{
			"や",
			"a",
			false,
			[]string{"ya"},
		},
		RomajiCharacter{
			"ら",
			"a",
			false,
			[]string{"ra"},
		},
		RomajiCharacter{
			"わ",
			"a",
			false,
			[]string{"wa"},
		},
		RomajiCharacter{
			"が",
			"a",
			true,
			[]string{"ga"},
		},
		RomajiCharacter{
			"ざ",
			"a",
			true,
			[]string{"za"},
		},
		RomajiCharacter{
			"だ",
			"a",
			true,
			[]string{"da"},
		},
		RomajiCharacter{
			"ば",
			"a",
			true,
			[]string{"ba"},
		},
		RomajiCharacter{
			"ぱ",
			"a",
			true,
			[]string{"pa"},
		},
	},
	"e": []RomajiCharacter{
		RomajiCharacter{
			"え",
			"e",
			false,
			[]string{"e"},
		},
		RomajiCharacter{
			"け",
			"e",
			false,
			[]string{"ke"},
		},
		RomajiCharacter{
			"せ",
			"e",
			false,
			[]string{"se"},
		},
		RomajiCharacter{
			"て",
			"e",
			false,
			[]string{"te"},
		},
		RomajiCharacter{
			"ね",
			"e",
			false,
			[]string{"ne"},
		},
		RomajiCharacter{
			"へ",
			"e",
			false,
			[]string{"he"},
		},
		RomajiCharacter{
			"め",
			"e",
			false,
			[]string{"me"},
		},
		RomajiCharacter{
			"れ",
			"e",
			false,
			[]string{"re"},
		},
		RomajiCharacter{
			"げ",
			"e",
			true,
			[]string{"ge"},
		},
		RomajiCharacter{
			"ぜ",
			"e",
			true,
			[]string{"ze"},
		},
		RomajiCharacter{
			"で",
			"e",
			true,
			[]string{"de"},
		},
		RomajiCharacter{
			"べ",
			"e",
			true,
			[]string{"be"},
		},
		RomajiCharacter{
			"ぺ",
			"e",
			true,
			[]string{"pe"},
		},
	},
	"i": []RomajiCharacter{
		RomajiCharacter{
			"い",
			"i",
			false,
			[]string{"i"},
		},
		RomajiCharacter{
			"き",
			"i",
			false,
			[]string{"ki"},
		},
		RomajiCharacter{
			"し",
			"i",
			false,
			[]string{"shi"},
		},
		RomajiCharacter{
			"ち",
			"i",
			false,
			[]string{"chi"},
		},
		RomajiCharacter{
			"に",
			"i",
			false,
			[]string{"ni"},
		},
		RomajiCharacter{
			"ひ",
			"i",
			false,
			[]string{"hi"},
		},
		RomajiCharacter{
			"み",
			"i",
			false,
			[]string{"mi"},
		},
		RomajiCharacter{
			"り",
			"i",
			false,
			[]string{"ri"},
		},
		RomajiCharacter{
			"ぎ",
			"i",
			true,
			[]string{"gi"},
		},
		RomajiCharacter{
			"じ",
			"i",
			true,
			[]string{"ji"},
		},
		RomajiCharacter{
			"ぢ",
			"i",
			true,
			[]string{"ji", "di"},
		},
		RomajiCharacter{
			"び",
			"i",
			true,
			[]string{"bi"},
		},
		RomajiCharacter{
			"ぴ",
			"i",
			true,
			[]string{"pi"},
		},
	},
	"o": []RomajiCharacter{
		RomajiCharacter{
			"お",
			"o",
			false,
			[]string{"o"},
		},
		RomajiCharacter{
			"こ",
			"o",
			false,
			[]string{"ko"},
		},
		RomajiCharacter{
			"そ",
			"o",
			false,
			[]string{"so"},
		},
		RomajiCharacter{
			"と",
			"o",
			false,
			[]string{"to"},
		},
		RomajiCharacter{
			"の",
			"o",
			false,
			[]string{"no"},
		},
		RomajiCharacter{
			"ほ",
			"o",
			false,
			[]string{"ho"},
		},
		RomajiCharacter{
			"も",
			"o",
			false,
			[]string{"mo"},
		},
		RomajiCharacter{
			"よ",
			"o",
			false,
			[]string{"yo"},
		},
		RomajiCharacter{
			"ろ",
			"o",
			false,
			[]string{"ro"},
		},
		RomajiCharacter{
			"を",
			"o",
			false,
			[]string{"o", "wo"},
		},
		RomajiCharacter{
			"ご",
			"o",
			true,
			[]string{"go"},
		},
		RomajiCharacter{
			"ぞ",
			"o",
			true,
			[]string{"zo"},
		},
		RomajiCharacter{
			"ど",
			"o",
			true,
			[]string{"do"},
		},
		RomajiCharacter{
			"ぼ",
			"o",
			true,
			[]string{"bo"},
		},
		RomajiCharacter{
			"ぽ",
			"o",
			true,
			[]string{"po"},
		},
	},
	"u": []RomajiCharacter{
		RomajiCharacter{
			"う",
			"u",
			false,
			[]string{"u"},
		},
		RomajiCharacter{
			"く",
			"u",
			false,
			[]string{"ku"},
		},
		RomajiCharacter{
			"す",
			"u",
			false,
			[]string{"su"},
		},
		RomajiCharacter{
			"つ",
			"u",
			false,
			[]string{"tsu"},
		},
		RomajiCharacter{
			"ぬ",
			"u",
			false,
			[]string{"nu"},
		},
		RomajiCharacter{
			"ふ",
			"u",
			false,
			[]string{"fu", "hu"},
		},
		RomajiCharacter{
			"む",
			"u",
			false,
			[]string{"mu"},
		},
		RomajiCharacter{
			"ゆ",
			"u",
			false,
			[]string{"yu"},
		},
		RomajiCharacter{
			"る",
			"u",
			false,
			[]string{"ru"},
		},
		RomajiCharacter{
			"ぐ",
			"u",
			true,
			[]string{"gu"},
		},
		RomajiCharacter{
			"ず",
			"u",
			true,
			[]string{"zu"},
		},
		RomajiCharacter{
			"づ",
			"u",
			true,
			[]string{"zu", "du", "dzu"},
		},
		RomajiCharacter{
			"ぶ",
			"u",
			true,
			[]string{"bu"},
		},
		RomajiCharacter{
			"ぷ",
			"u",
			true,
			[]string{"pu"},
		},
	},
	"n": []RomajiCharacter{
		RomajiCharacter{
			"ん",
			"n",
			false,
			[]string{"n"},
		},
	},
}

var ROMAJI_KATAKANA = map[string][]RomajiCharacter{
	"a": []RomajiCharacter{
		RomajiCharacter{
			"ア",
			"a",
			false,
			[]string{"a"},
		},
		RomajiCharacter{
			"ハ",
			"a",
			false,
			[]string{"ha"},
		},
		RomajiCharacter{
			"バ",
			"a",
			true,
			[]string{"ba"},
		},
		RomajiCharacter{
			"パ",
			"a",
			true,
			[]string{"pa"},
		},
		RomajiCharacter{
			"カ",
			"a",
			false,
			[]string{"ka"},
		},
		RomajiCharacter{
			"ガ",
			"a",
			true,
			[]string{"ga"},
		},
		RomajiCharacter{
			"タ",
			"a",
			false,
			[]string{"ta"},
		},
		RomajiCharacter{
			"ダ",
			"a",
			true,
			[]string{"da"},
		},
		RomajiCharacter{
			"サ",
			"a",
			false,
			[]string{"sa"},
		},
		RomajiCharacter{
			"ザ",
			"a",
			true,
			[]string{"za"},
		},
		RomajiCharacter{
			"ナ",
			"a",
			false,
			[]string{"na"},
		},
		RomajiCharacter{
			"マ",
			"a",
			false,
			[]string{"ma"},
		},
		RomajiCharacter{
			"ラ",
			"a",
			false,
			[]string{"ra"},
		},
		RomajiCharacter{
			"ワ",
			"a",
			false,
			[]string{"wa"},
		},
		RomajiCharacter{
			"ヤ",
			"a",
			false,
			[]string{"ya"},
		},
		RomajiCharacter{
			"ヷ",
			"a",
			true,
			[]string{"va"},
		},
	},
	"i": []RomajiCharacter{
		RomajiCharacter{
			"イ",
			"i",
			false,
			[]string{"i"},
		},
		RomajiCharacter{
			"ヒ",
			"i",
			false,
			[]string{"hi"},
		},
		RomajiCharacter{
			"ビ",
			"i",
			true,
			[]string{"bi"},
		},
		RomajiCharacter{
			"ピ",
			"i",
			true,
			[]string{"pi"},
		},
		RomajiCharacter{
			"キ",
			"i",
			false,
			[]string{"ki"},
		},
		RomajiCharacter{
			"ギ",
			"i",
			true,
			[]string{"gi"},
		},
		RomajiCharacter{
			"チ",
			"i",
			false,
			[]string{"chi"},
		},
		RomajiCharacter{
			"ヂ",
			"i",
			true,
			[]string{"di", "ji"},
		},
		RomajiCharacter{
			"シ",
			"i",
			false,
			[]string{"shi"},
		},
		RomajiCharacter{
			"ジ",
			"i",
			true,
			[]string{"zi", "ji"},
		},
		RomajiCharacter{
			"ニ",
			"i",
			false,
			[]string{"ni"},
		},
		RomajiCharacter{
			"ミ",
			"i",
			false,
			[]string{"mi"},
		},
		RomajiCharacter{
			"リ",
			"i",
			false,
			[]string{"ri"},
		},
	},
	"u": []RomajiCharacter{
		RomajiCharacter{
			"ウ",
			"u",
			false,
			[]string{"u"},
		},
		RomajiCharacter{
			"フ",
			"u",
			false,
			[]string{"hu", "fu"},
		},
		RomajiCharacter{
			"ブ",
			"u",
			true,
			[]string{"bu"},
		},
		RomajiCharacter{
			"プ",
			"u",
			true,
			[]string{"pu"},
		},
		RomajiCharacter{
			"ク",
			"u",
			false,
			[]string{"ku"},
		},
		RomajiCharacter{
			"グ",
			"u",
			true,
			[]string{"gu"},
		},
		RomajiCharacter{
			"ツ",
			"u",
			false,
			[]string{"tsu"},
		},
		RomajiCharacter{
			"ヅ",
			"u",
			true,
			[]string{"zu", "du", "dzu"},
		},
		RomajiCharacter{
			"ス",
			"u",
			false,
			[]string{"su"},
		},
		RomajiCharacter{
			"ズ",
			"u",
			true,
			[]string{"zu"},
		},
		RomajiCharacter{
			"ヌ",
			"u",
			false,
			[]string{"nu"},
		},
		RomajiCharacter{
			"ム",
			"u",
			false,
			[]string{"mu"},
		},
		RomajiCharacter{
			"ル",
			"u",
			false,
			[]string{"ru"},
		},
		RomajiCharacter{
			"ユ",
			"u",
			false,
			[]string{"yu"},
		},
		RomajiCharacter{
			"ヴ",
			"u",
			true,
			[]string{"vu", "bu"},
		},
	},
	"e": []RomajiCharacter{
		RomajiCharacter{
			"エ",
			"e",
			false,
			[]string{"e"},
		},
		RomajiCharacter{
			"ヘ",
			"e",
			false,
			[]string{"he"},
		},
		RomajiCharacter{
			"ベ",
			"e",
			true,
			[]string{"be"},
		},
		RomajiCharacter{
			"ペ",
			"e",
			true,
			[]string{"pe"},
		},
		RomajiCharacter{
			"ケ",
			"e",
			false,
			[]string{"ke"},
		},
		RomajiCharacter{
			"ゲ",
			"e",
			true,
			[]string{"ge"},
		},
		RomajiCharacter{
			"テ",
			"e",
			false,
			[]string{"te"},
		},
		RomajiCharacter{
			"デ",
			"e",
			true,
			[]string{"de"},
		},
		RomajiCharacter{
			"セ",
			"e",
			false,
			[]string{"se"},
		},
		RomajiCharacter{
			"ゼ",
			"e",
			true,
			[]string{"ze"},
		},
		RomajiCharacter{
			"ネ",
			"e",
			false,
			[]string{"ne"},
		},
		RomajiCharacter{
			"メ",
			"e",
			false,
			[]string{"me"},
		},
		RomajiCharacter{
			"レ",
			"e",
			false,
			[]string{"re"},
		},
	},
	"o": []RomajiCharacter{
		RomajiCharacter{
			"オ",
			"o",
			false,
			[]string{"o"},
		},
		RomajiCharacter{
			"ホ",
			"o",
			false,
			[]string{"ho"},
		},
		RomajiCharacter{
			"ボ",
			"o",
			true,
			[]string{"bo"},
		},
		RomajiCharacter{
			"ポ",
			"o",
			true,
			[]string{"po"},
		},
		RomajiCharacter{
			"コ",
			"o",
			false,
			[]string{"ko"},
		},
		RomajiCharacter{
			"ゴ",
			"o",
			true,
			[]string{"go"},
		},
		RomajiCharacter{
			"ト",
			"o",
			false,
			[]string{"to"},
		},
		RomajiCharacter{
			"ド",
			"o",
			true,
			[]string{"do"},
		},
		RomajiCharacter{
			"ソ",
			"o",
			false,
			[]string{"so"},
		},
		RomajiCharacter{
			"ゾ",
			"o",
			true,
			[]string{"zo"},
		},
		RomajiCharacter{
			"ノ",
			"o",
			false,
			[]string{"no"},
		},
		RomajiCharacter{
			"モ",
			"o",
			false,
			[]string{"mo"},
		},
		RomajiCharacter{
			"ロ",
			"o",
			false,
			[]string{"ro"},
		},
		RomajiCharacter{
			"ヲ",
			"o",
			false,
			[]string{"wo"},
		},
		RomajiCharacter{
			"ヨ",
			"o",
			false,
			[]string{"yo"},
		},
		RomajiCharacter{
			"ヺ",
			"o",
			true,
			[]string{"vo"},
		},
	},
	"n": []RomajiCharacter{
		RomajiCharacter{
			"ン",
			"n",
			false,
			[]string{"n"},
		},
	},
}

